package org.yaoqiang.bpmn.model.elements.choreographyactivities;

import java.util.ArrayList;
import java.util.List;

import org.yaoqiang.bpmn.model.elements.XMLElement;
import org.yaoqiang.bpmn.model.elements.artifacts.Artifacts;
import org.yaoqiang.bpmn.model.elements.core.common.FlowElement;
import org.yaoqiang.bpmn.model.elements.core.common.FlowElements;
import org.yaoqiang.bpmn.model.elements.core.common.FlowElementsContainer;
import org.yaoqiang.bpmn.model.elements.core.common.FlowNode;
import org.yaoqiang.bpmn.model.elements.core.common.SequenceFlow;

/**
 * SubChoreography
 * 
 * @author Shi Yaoqiang(shi_yaoqiang@yahoo.com)
 */
public class SubChoreography extends ChoreographyActivity implements FlowElementsContainer {

	public SubChoreography(FlowElements parent) {
		super(parent, "subChoreography");
	}

	protected void fillStructure() {
		FlowElements refFlowElements = new FlowElements(this);
		Artifacts refArtifacts = new Artifacts(this);
		
		super.fillStructure();
		add(refFlowElements);
		add(refArtifacts);
	}

	public final FlowElement getFlowElement(String id) {
		return (FlowElement) getFlowElements().getCollectionElement(id);
	}
	
	public final List<XMLElement> getFlowElementList() {
		return getFlowElements().getXMLElements();
	}

	public final FlowElements getFlowElements() {
		return (FlowElements) get("flowElements");
	}
	
	public final List<FlowNode> getFlowNodes() {
		List<FlowNode> flowNodes = new ArrayList<FlowNode>();
		for (XMLElement flowElement : getFlowElementList()) {
			if (flowElement instanceof FlowNode) {
				flowNodes.add((FlowNode) flowElement);
			}
		}
		return flowNodes;
	}
	
	public final List<SequenceFlow> getSequenceFlows() {
		List<SequenceFlow> sequenceFlows = new ArrayList<SequenceFlow>();
		for (XMLElement flowElement : getFlowElementList()) {
			if (flowElement instanceof SequenceFlow) {
				sequenceFlows.add((SequenceFlow) flowElement);
			}
		}
		return sequenceFlows;
	}
	
	public final Artifacts getArtifacts() {
		return (Artifacts) get("artifacts");
	}
	
	public final XMLElement generateFlowElement(String type) {
		FlowElements flowElements = getFlowElements();
		flowElements.setType(type);
		XMLElement el = flowElements.generateNewElement();
		return el;
	}
	
	public final void addFlowElement(XMLElement el) {
		getFlowElements().add(el);
	}
	
}
